%lamp_model
% Model of the V vs I curve of a filament lamp. The voltage is raised (and
% then lowered) in a set time (tor) between fixed limits set by a potential
% divider circuit.
%
% LAST UPDATED by Andy French September 2017

function lamp_model

%% Inputs %%

%Stefan's constant for radiation /Wm^-1K^-1
sigma_stefan = 5.67e-8;

%Emmissivity
em = 1;

%Mass of tungsten filament /g
m = 0.02;

%Specific heat capacity of tungsten filament /J per gram /K
c = 0.134;

%Surface area of filament /m^2
A = 5e-5;

%Range of tor values /s. Same tor for lighting, and tor for dimming
tor = 5:30;

%Current /amps and voltage /volts measurements at extreme ends.
%Assume equilibrium has been reached in both cases.
V0 = 0.2;
I0 = 75e-3;
Vmax = 1.8;
Imax = 210e-3;

%V,I limits for graph (volts, amps respectively)
Vgraphmax = 2;
Igraphmax = 0.3;

%Number of t values to compute lighting and dimming steps
N = 1000;

%Fontsize for graphs
fsize = 18;

%

%% RUN MODEL AND PLOT CURVES %%

%Plot V vs I curves
figure('name','lamp V vs I curve','color',[ 1 1 1 ] )
hold on;
for n=1:length(tor)
    %Determine ligting and dimming curves
    [I,V,R,T,t,lightordim,R0,alpha,T0] = lampcalc( sigma_stefan,em,m,c,A,tor(n),V0,I0,Vmax,Imax,N,'light' );
    
    %Determine line colour
    if max(tor)==min(tor)
        color = [0,0,1];
    else
        f = ( tor(n) - min(tor) )/( max(tor) - min(tor) );
        color = [1,0,0]*(1-f) + [0,0,1]*f;
    end
    
    %Plot I,V curve
    plot( I,V,'-','color',color );
end
xlabel( 'Current /amps','fontsize',fsize );
ylabel( 'Voltage /volts','fontsize',fsize );
title( ['Filament lamp V,I curve. \tau from ',num2str(min(tor)),'s (red) to ',num2str(max(tor)),'s (blue)'],'fontsize',fsize );
set(gca,'fontsize',fsize)
box on;
grid on;
xlim([0,Igraphmax]);
ylim([0,Vgraphmax]);
print( gcf, 'lamp IV.png','-dpng','-r300');

%

%Plot temperature vs time to determine time required to reach equilibrium,
%resulting from the final T and V values.
figure('name','lamp final T vs t','color',[ 1 1 1 ] )
hold on;
for n=1:length(tor)
    
    %Determine ligting and dimming curves
    [I,V,R,T,t,lightordim,R0,alpha,T0] = lampcalc( sigma_stefan,em,m,c,A,tor(n),V0,I0,Vmax,Imax,N,'light' );
    
    %Determine T vs t after V0 has been reached again
    [t_eq,tt,TT] = constant_V_time_to_eq(T(end),V(end),m,c,R0,alpha,T0,sigma_stefan,em,A);
    
    %Determine colour of line
    if max(tor)==min(tor)
        color = [0,0,1];
    else
        f = ( tor(n) - min(tor) )/( max(tor) - min(tor) );
        color = [1,0,0]*(1-f) + [0,0,1]*f;
    end
    
    %Plot time vs temperature curve
    plot(t,T,'linestyle','-','color',color);
    plot(t(end)+tt,TT,'linestyle','--','color',color);
end
legend({'Light then dim','Fixed at V_0'},'fontsize',fsize);
xlabel('time /s','fontsize',fsize);
ylabel('Temperature /K','fontsize',fsize);
set(gca,'fontsize',fsize);
xlim([0,t(end)+t_eq])
box on
grid on;
title(['Filament temperature. \tau from ',num2str(min(tor)),...
    's (red) to ',num2str(max(tor)),'s (blue).  V_0 = ',num2str(V0),'volts'],'fontsize',fsize);
print( gcf, 'T vs time.png','-dpng','-r300');

%

%Plot filament resistance vs time
figure('name','lamp R vs t','color',[ 1 1 1 ] )
hold on;
for n=1:length(tor)
    
    %Determine ligting and dimming curves
    [I,V,R,T,t,lightordim,R0,alpha,T0] = lampcalc( sigma_stefan,em,m,c,A,tor(n),V0,I0,Vmax,Imax,N,'light' );
    
    %Determine R vs t after V0 has been reached again
    [t_eq,tt,TT] = constant_V_time_to_eq(T(end),V(end),m,c,R0,alpha,T0,sigma_stefan,em,A);
    [RR,R0,alpha] = resistance_from_T( TT, sigma_stefan,em,m,c,A,tor,V0,I0,Vmax,Imax );
    
    %Determine colour of line
    if max(tor)==min(tor)
        color = [0,0,1];
    else
        f = ( tor(n) - min(tor) )/( max(tor) - min(tor) );
        color = [1,0,0]*(1-f) + [0,0,1]*f;
    end
    
    %Plot time vs resistance curve
    plot(t,R,'linestyle','-','color',color);
    plot(t(end)+tt,RR,'linestyle','--','color',color);
end
legend({'Light then dim','Fixed at V_0'},'fontsize',fsize);
xlabel('time /s','fontsize',fsize);
ylabel('Resistance /\Omega','fontsize',fsize);
set(gca,'fontsize',fsize);
xlim([0,t(end)+t_eq])
box on
grid on;
title(['Filament resistance. \tau from ',num2str(min(tor)),...
    's (red) to ',num2str(max(tor)),'s (blue).  V_0 = ',num2str(V0),'volts'],'fontsize',fsize);
print( gcf, 'R vs time.png','-dpng','-r300');

%%

%Plot resistance vs T line
figure('name','lamp R vs T','color',[ 1 1 1 ] )
plot(T,R);
xlabel('Temperature /K','fontsize',fsize);
ylabel('Resistance /\Omega','fontsize',fsize);
set(gca,'fontsize',fsize);
box on
grid on;
title({'Filament resistance vs temperature',...
    ['R = ',num2str(R0),' + ',num2str(alpha),'( T - ',num2str(T0),' )']},'fontsize',fsize);
print( gcf, 'R vs temp.png','-dpng','-r300');

%%

%% SUB-FUNCTIONS %%

%Numeric model to caluclate I,V,T,t curves
function [I,V,R,T,t,lightordim,R0,alpha,T0] = lampcalc( sigma_stefan,em,m,c,A,tor,V0,I0,Vmax,Imax,N,light_or_dim )

%Times /s
t = linspace(0,2*tor,2*N);

%Set lightordim flags. (1 means lighting).
lightordim = [ones(1,N),zeros(1,N) ];

%Determine minimum filament temperature /K. Assumed at equilibrium when the power input
%equals the radiant heat produced.
T0 = (I0*V0/(em*sigma_stefan*A))^(1/4);

%Initialize temperature array /K
T = T0*ones(1,2*N);

%Determine resistance (/ohms) vs temperature (K) linear coefficient, and R0
[R,R0,alpha] = resistance_from_T( T0*ones(1,2*N), sigma_stefan,em,m,c,A,tor,V0,I0,Vmax,Imax );

%Voltage across lamp /volts
V(1:N) = V0 + (Vmax - V0)*t(1:N)/tor;
V(N+1:2*N) = Vmax - (Vmax - V0)*( t(N+1:2*N) - tor )/tor;

%Numerically work out parameters vs time t using a Euler method
dt = tor/(N-1);
for n=2:2*N
    %Determine temperature change from heat balance equation
    dT = ( V(n-1)*V(n-1)/R(n-1) - em*sigma_stefan*A*(T(n-1)^4) )*dt/(m*c);
    
    %Update temperature and resistance vectors
    T(n) = T(n-1) + dT;
    R(n) = R0 + alpha*( T(n) - T0 );
end

%Apply Ohm's law to calculate current I /amps
I = V./R;

%%

%Function to determine T vs t curve for starting T,V values, and hence time
%taken to reach equilibrium t_eq. Assumes constant voltage vs time.
function [t_eq,t,T] = constant_V_time_to_eq(T1,V,m,c,R0,alpha,T0,sigma_stefan,em,A)

%Work out N temperature and time points
nmax = 10000;
dTmin = 0.001;
dt = 0.01;
T = T1;
t = 0;
dT = V^2/(R0 + alpha*(T1-T0)) - sigma_stefan*em*A*T0^4;
n=1;
while ( abs(dT)>dTmin ) && (n<nmax)
    TT = T(end);
    dT = (V^2)/(R0 + alpha*(TT-T0)) - sigma_stefan*em*A*TT^4;
    T = [T,TT+dT];
    t = [t,t(end)+dt];
    n = n + 1;
end
t_eq = t(end);

%%

%Function to determine Resistance vs Temperature for the filament lamp.
%R = R0 + alpha(T-T0). Note V0 = I0*R0. This could be improved by
%making several equilibrium I,V measurements and performing a line of best
%fit analysis to V/I vs [IV/(sigma_stefan*em*A)]^(1/4).
function [R,R0,alpha] = resistance_from_T( T, sigma_stefan,em,m,c,A,tor,V0,I0,Vmax,Imax )

%Determine maximum and minimum filament temperature /K i.e. at equilibrium when the power input
%equals the radiant heat produced.
Tmax = (Imax*Vmax/(em*sigma_stefan*A))^(1/4);
T0 = (I0*V0/(em*sigma_stefan*A))^(1/4);

%Determine resistance (/ohms) vs temperature coefficient and R0
R0 = V0/I0;
alpha = ( (em*sigma_stefan*A)^(1/4) )*(Vmax/Imax - V0/I0)/...
    ( (Imax*Vmax)^(1/4) - (I0*V0)^(1/4) );

%Calculate resistance /ohms from T in kelvin
R = R0 + alpha*( T - T0 );

%End of code