%lamp_IV_spoon
% Function which performs the following:
%
% 1  Loads IV data taken in steps at thermal equilibrium and
%    uses this to determine Resistance vs Temperature for a
%    lamp of known filament area A.
%
% 2  Loads IV data for V shaped voltage sweep vs time (six runs).
%    For each run, use a heat balance model to predict temperature
%    and hence resistance and current. Plot against IV data.
%    Have the option of using a linear voltage sweep. Compute
%    the average rate of change of voltage from the data by
%    fitting a regression line, separately, to the up and down
%    sweeps.
%
% 3  For each figure, use a key press handler to vary (i) the
%    filament mass and (ii) the filament radiating area.
%    These two parameters are unknown, and can be varied to fit
%    the data.
%
% LAST UPDATED by Andy French September 2017

function lamp_IV_spoon(varargin)
global d

%% INPUTS %%

%Choose experimental run
if nargin==0
    d.experiment_run = 1;
else
    d.experiment_run = varargin{1};
end

%Mass of tungsten filament /g
d.M = 0.001;
d.dM = d.M/10;

%Surface area of filament /m^2
d.A = 9e-6;
d.dA = d.A/10;

%Stefan's constant for radiation /Wm^-1K^-1
d.sigma_stefan = 5.67e-8;

%Specific heat capacity of tungsten filament /J per gram /K
d.specific_heat_capacity = 0.134;

%V,I limits for graph (volts, amps respectively)
Vgraphmax = 2;
Igraphmax = 0.25;

%Number of t values to compute lighting and dimming steps in model
d.N = 1000;

%Fontsize for graphs
fsize = 18;

%

%% Load data and set up blank graph %%

%Load lamp IV data, that has been ingested from an Excel sheet
load Lamp_IV_data

%Create a blank figure
d.fig = figure('name','lamp IV experiment','color',[1 1 1]);

%

%% Plot graphs of stepped equilibrium data %%

%Voltage vs time for stepped equilibrium
subplot(2,2,1);
plot( t_RvsT, V_RvsT, 'r' );
xlabel('time /s','fontsize',fsize)
ylabel('Voltage /volts','fontsize',fsize)
title('Lamp voltage vs time','fontsize',fsize)
xlim([0,max(t_RvsT)])
grid on
set(gca,'fontsize',fsize)
box on;

%Current vs time for stepped equilibrium
subplot(2,2,2);
plot( t_RvsT, I_RvsT, 'r' );
xlabel('time /s','fontsize',fsize)
ylabel('Current /amps','fontsize',fsize)
title('Lamp current vs time','fontsize',fsize)
xlim([0,max(t_RvsT)])
grid on
set(gca,'fontsize',fsize)
box on;

%Resistance vs time for stepped equilibrium
subplot(2,2,3);
plot( t_RvsT, V_RvsT./I_RvsT, 'r' );
xlabel('time /s','fontsize',fsize)
ylabel('Resistance /\Omega','fontsize',fsize)
title('Lamp resistance vs time','fontsize',fsize)
xlim([0,max(t_RvsT)])
grid on
set(gca,'fontsize',fsize)
box on;

%Power vs time for stepped equilibrium
subplot(2,2,4);
plot( t_RvsT, I_RvsT.*V_RvsT, 'm' );
xlabel('time /s','fontsize',fsize)
ylabel('Power /watts','fontsize',fsize)
title('Lamp power vs time','fontsize',fsize)
xlim([0,max(t_RvsT)])
grid on
set(gca,'fontsize',fsize)
box on;

%Save PNG file
print(d.fig, 'R vs t\stepped eq I V R P vs time.png','-dpng','-r300')
clf;

%

%% Determine R vs T for lamp %%

%Assuming balance of input power (IV) to radiation (sigma_stefan*A*T^4)
%then a graph of R=V/I vs (IV/sigma_stefan)^0.25 should yield a straight line if
%resistance R varies linearly with temperature. R = R0 + alpha*T
%
% If y = V/I and x = (IV/sigma_stefan)^0.25, then y = mx + c
% alpha/(A^0.25) = m  therefore alpha = m*A^0.25

%Form x,y vectors
x = (I_RvsT.*V_RvsT/d.sigma_stefan).^0.25;
y = V_RvsT./I_RvsT;

%Determine line of best fit
[yfit,d.m,c,p] = bestfit(x,y);

%Plot V/I vs (IV/\sigma)^0.25 and overlay with line of best fit
plot(x,y,'r+',x,yfit,'k-');
xlabel('(IV/\sigma)^{0.25}','fontsize',fsize)
ylabel('R = V/I','fontsize',fsize)
title({'Resistance vs (IV/\sigma)^{0.25}',...
    ['y = ',num2str(c),' + ',num2str(d.m),'x, p = ',num2str(p)]},'fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
box on;

%Save PNG file
print(d.fig, 'R vs t\R vs fourth root of IV.png','-dpng','-r300')
clf;

%

%Compute temperatures, assuming equilibrium at all points (i.e. we ignore
%the transitions between the voltage and current steps)
R0 = c;
d.alpha = d.m*(d.A^0.25);
R_RvsT = V_RvsT./I_RvsT;
T_RvsT = (I_RvsT.*V_RvsT/(d.sigma_stefan*d.A)).^0.25;

%Plot V/I vs T and overlay with line of best fit
plot(T_RvsT,R_RvsT,'r+',T_RvsT, R0 + d.alpha*T_RvsT,'k-');
xlabel('Temperature /K','fontsize',fsize)
ylabel('R /\Omega','fontsize',fsize)
title({'Resistance vs temperature',...
    ['R = ',num2str(R0),' + ',num2str(d.alpha),'T',]},'fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
box on;

%Save PNG file
print(d.fig, 'R vs t\R vs T.png','-dpng','-r300')
clf;

%

%% Load IV data for lamp and compare to model %%

% Get run IV data
if d.experiment_run==1
    t = t1; V = V1; I = I1;
elseif d.experiment_run==2
    t = t2; V = V2; I = I2;
elseif d.experiment_run==3
    t = t3; V = V3; I = I3;
elseif d.experiment_run==4
    t = t4; V = V4; I = I4;
elseif d.experiment_run==5
    t = t5; V = V5; I = I5;
else
    t = t1; V = V1; I = I1;
end

%Determine resistance /ohms and temperature /K
R = V./I;
T = (R - R0)/d.alpha;

%Determine lines of best fit to each sides of voltage sweep
d.Vmax = max(V);
d.Vmin = V(1);
d.Imin = I(1);
imax = find( V == d.Vmax ); imax = imax(1);
NN = length(t);
[yfit_up,m_up,c_up,p_up] = bestfit(t(1:imax),V(1:imax));
[yfit_down,m_down,c_down,p_down] = bestfit( t(imax:NN),V(imax:NN) );

%Calculate average voltage sweep
d.tor = 0.5*(d.Vmax-d.Vmin)/m_up + 0.5*(d.Vmax-d.Vmin)/abs(m_down);

%Plot V vs t
plot(t,V,'r+'); hold on;
plot(t(1:imax),yfit_up,'k-','linewidth',2);
plot(t(imax:NN),yfit_down,'k-','linewidth',2);
xlabel('time /s','fontsize',fsize)
ylabel('Voltage /volts','fontsize',fsize)
title(['V vs t for Run #',num2str(d.experiment_run),...
    ': \tau = ',num2str(d.tor,4),'s'],'fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
box on;
xlim([0,max(t)]);
ylim([0,Vgraphmax]);

%Save PNG file
print(d.fig, ['V vs t curves\Run ',num2str(d.experiment_run),' V vst t.png'],'-dpng','-r300')
clf;

%

%Overlay a model curve with the I,V data. The m prefix means model
[mt,mV,mI,mR,mT] = spoon( d.tor, d.Vmin,d.Vmax,d.Imin,d.sigma_stefan,...
    d.A, d.M, d.specific_heat_capacity, d.alpha, d.N );

%Plot R vs t
subplot(2,1,1);
plot(t,R,'r+',mt,mR,'b-');
xlabel('Time /s','fontsize',fsize)
ylabel('Resistance /\Omega','fontsize',fsize)
d.IVtitle = title({['R vs t curve for Run #',num2str(d.experiment_run)]...
    ['A = ',num2str(d.A,4),'m^2, M = ',...
    num2str(d.M,4),'g, \tau = ',num2str(d.tor,4),'s'] },'fontsize',fsize);
legend({'Data','Model'},'fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
box on;
xlim([0,max(t)])

%Plot T vs t
subplot(2,1,2);
plot(t,T,'r+',mt,mT,'b-');
xlabel('Time /s','fontsize',fsize)
ylabel('Filament temperature /K','fontsize',fsize)
d.IVtitle = title({['T vs t curve for Run #',num2str(d.experiment_run)]...
    ['A = ',num2str(d.A,4),'m^2, M = ',...
    num2str(d.M,4),'g, \tau = ',num2str(d.tor,4),'s'] },'fontsize',fsize);
legend({'Data','Model'},'fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
box on;
xlim([0,max(t)])

%Save PNG file
print(d.fig, ['R & T vs t curves\Run ',num2str(d.experiment_run),' R&T vs t.png'],'-dpng','-r300')
clf;

%

%Plot V vs I
plot(I,V,'r+'); hold on;
d.model = plot(mI,mV,'b');
xlabel('Current /amps','fontsize',fsize)
ylabel('Voltage /volts','fontsize',fsize)
d.IVtitle = title({['V vs I curve for Run #',num2str(d.experiment_run)]...
    ['A = ',num2str(d.A,4),'m^2, M = ',...
    num2str(d.M,4),'g, \tau = ',num2str(d.tor,4),'s'] },'fontsize',fsize);
legend({'Data','Model'},'fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
box on;
ylim([0,Vgraphmax])
xlim([0,Igraphmax])

%Save PNG file
print(d.fig, ['IV curves\Run ',num2str(d.experiment_run),' IV.png'],'-dpng','-r300')

%Set key-press handler for graph to enable A and m to be modified via
%arrow keys
set( d.fig,'KeyPressFcn',@keypressfunc );

%%

%Key press handler
function keypressfunc( fig,evnt )
global d

%Get the string of the current key press
k = get( d.fig,'currentkey');

%Actions, depending on the key pressed...
if strcmp(k,'q') == 1
    %Quit
    close(d.fig);
elseif strcmp(k,'uparrow') == 1
    %Increment filament area
    d.A = d.A + d.dA;
    d.alpha = d.m*(d.A^0.25);
    [mt,mV,mI,mR,mT] = spoon( d.tor, d.Vmin,d.Vmax,d.Imin,d.sigma_stefan,...
        d.A, d.M, d.specific_heat_capacity, d.alpha, d.N );
    set(d.model,'xdata',mI,'ydata',mV);
    set(d.IVtitle,'string',{['V vs I curve for Run #',num2str(d.experiment_run)]...
        ['A = ',num2str(d.A,4),'m^2, M = ',num2str(d.M,4),'g, \tau = ',num2str(d.tor,4),'s'] } );
elseif strcmp(k,'rightarrow') == 1
    %Increment filament mass
    d.M = d.M + d.dM;
    [mt,mV,mI,mR,mT] = spoon( d.tor, d.Vmin,d.Vmax,d.Imin,d.sigma_stefan,...
        d.A, d.M, d.specific_heat_capacity, d.alpha, d.N );
    set(d.model,'xdata',mI,'ydata',mV);
    set(d.IVtitle,'string',{['V vs I curve for Run #',num2str(d.experiment_run)]...
        ['A = ',num2str(d.A,4),'m^2, M = ',num2str(d.M,4),'g, \tau = ',num2str(d.tor,4),'s'] } );
elseif strcmp(k,'leftarrow') == 1
    %Decrement filament area
    d.M = d.M - d.dM;
    if d.M <= 0
        d.M = d.dM;
    end
    [mt,mV,mI,mR,mT] = spoon( d.tor, d.Vmin,d.Vmax,d.Imin,d.sigma_stefan,...
        d.A, d.M, d.specific_heat_capacity, d.alpha, d.N );
    set(d.model,'xdata',mI,'ydata',mV);
    set(d.IVtitle,'string',{['V vs I curve for Run #',num2str(d.experiment_run)]...
        ['A = ',num2str(d.A,4),'m^2, M = ',num2str(d.M,4),'g, \tau = ',num2str(d.tor,4),'s'] } );
elseif strcmp(k,'downarrow') == 1
    %Decrement filament area
    d.A = d.A - d.dA;
    if d.A <= 0
        d.A = d.dA;
    end
    d.alpha = d.m*(d.A^0.25);
    [mt,mV,mI,mR,mT] = spoon( d.tor, d.Vmin,d.Vmax,d.Imin,d.sigma_stefan,...
        d.A, d.M, d.specific_heat_capacity, d.alpha, d.N );
    set(d.model,'xdata',mI,'ydata',mV);
    set(d.IVtitle,'string',{['V vs I curve for Run #',num2str(d.experiment_run)]...
        ['A = ',num2str(d.A,4),'m^2, M = ',num2str(d.M,4),'g, \tau = ',num2str(d.tor,4),'s'] } );
elseif strcmp(k,'p') == 1
    %Print new figure
    print(d.fig, ['IV curves\Run ',num2str(d.experiment_run),' IV.png'],'-dpng','-r300')
end

%%

%spoon
% Mathematical model of lamp current, temperature and resistance assuming
% a symmetric linear voltage sweep from Vmin to Vmax in time tor, and then
% Vmax to Vmin also in time tor. N specifies the number of model data points.
function [t,V,I,R,T] = spoon( tor, Vmin,Vmax,Imin,sigma_stefan, A, M, c, alpha, N )

%Determine initial temperature /K
T0 = ( Vmin*Imin/(sigma_stefan*A)).^0.25;

%Initialize outputs
V = [ linspace(Vmin,Vmax,N),linspace(Vmax,Vmin,N) ];
t = linspace(0,2*tor,2*N);
I = Imin*ones(1,2*N);
T = T0*ones(1,2*N);
R = (Vmin/Imin)*ones(1,2*N);

%Step through t vector and determine changes in T, R, I
dt = 2*tor/(2*N-1);
for n=2:2*N
    dT = dt*( I(n-1)*V(n-1) - sigma_stefan * A * T(n-1)^4 )/(M*c);
    T(n) = T(n-1) + dT;
    R(n) = R(n-1) + alpha*dT;
    I(n) = V(n)/R(n);
end

%%

%Line of best fit calculator
function [yfit,m,c,p] = bestfit(x,y)
xbar = mean(x);
ybar = mean(y);
xybar = mean(x.*y);
cov_xy = xybar - xbar*ybar;
m = cov_xy/var(x);
c = ybar - m*xbar;
p = cov_xy/sqrt( var(x)*var(y) );
yfit = m*x + c;

%End of code